#!/usr/bin/env python

import os
import sys

from setuptools import setup, Command
from setuptools.command.test import test as TestCommand
import pkg_resources


def read(readme_file):
    return open(os.path.join(os.path.dirname(__file__), readme_file)).read()


setup(
    name='pokr',
    version='0.0.0',
    author='Ross Fenning',
    author_email='pypi@rossfenning.co.uk',
    packages=['pokr'],
    description='Framework for building product and personal 2scorecards.',
    url='https://github.com/avengerpenguin/pokr',
    install_requires=[
        'quart', 'invoke', 'beautifulsoup4', 'PyGithub', 'todoist-python', 'sh', 'cachetools', 'livereload',
        'google-api-python-client', 'google-auth-httplib2', 'google-auth-oauthlib', 'feedparser', 'pybraries',
    ],
    tests_require=[],
    setup_requires=[],
)



import asyncio
import os
from typing import Text, Dict, Union, Callable, Coroutine

from invoke import task, Collection, Task
from quart import Quart, render_template

from . import metrics

__all__ = ["app", "invoke", "metrics"]

from .metrics import Metric


async def task_tuple(name: Text, coro: Callable[[], Coroutine]):
    return name, await coro()


def app(
    name: Text,
    metric_functions: Dict[Text, Dict[Text, Callable[[], Coroutine]]] = None
) -> Quart:
    template_dir = os.path.join(
        os.path.abspath(os.path.dirname(__file__)),
        'templates')
    quart_app = Quart(name, template_folder=template_dir)
    quart_app.config.from_mapping(debug=True)

    @quart_app.route("/")
    async def index() -> str:
        scorecard = {}

        for heading, kpis in metric_functions.items():
            scorecard[heading] = []

            tasks = []
            for kpi, gen in kpis.items():
                tasks.append(asyncio.create_task(task_tuple(kpi, gen)))

            for coro in asyncio.as_completed(tasks):
                metric_name, metric = await coro

                scorecard[heading].append(dict(name=metric_name, **metric))

        return await render_template("scorecard.html", scorecard=scorecard)

    return quart_app


def add_task(collection: Collection, t: Task, **project_args) -> None:
    @task(
        name=t.name, optional=t.optional,
    )
    def wrapped_task(c, **task_args):
        return t(c, **project_args, **task_args)

    wrapped_task.__doc__ = t.__doc__
    collection.add_task(wrapped_task, name=t.__name__)


def invoke() -> Collection:
    from . import tasks as t
    collection = Collection("tasks")
    add_task(collection, t.livereload)
    return collection
